<?php

namespace GoalTrackerPatreon;

defined( 'ABSPATH' ) or die;

class Options {

	const page_slug = 'goals_for_patreon_options';
	const gtgp_options_field_group_name = 'goals_for_patreon_options_group';

	const creator_info_section_name = 'goals_for_patreon_creator_section';

	// set by the user
	const client_secret_option_name = 'goals_for_patreon_client_secret';
	const client_id_option_name = 'goals_for_patreon_client_id';
	
	// initially set by user, then updated by Patreon
	const refresh_token_option_name = 'goals_for_patreon_creator_refresh_token';
	const access_token_option_name = 'goals_for_patreon_creator_access_token';

	// provided by Patreon
	const creator_id_option_name = 'goals_for_patreon_creator_id';
	const access_token_expire_time_option_name = 'goals_for_patreon_access_token_expires';

	// dsplay options
	const display_options_section_name = 'goals_for_patreon_display_section';
	const display_color_scheme_option_name = 'goals_for_patreon_display_color_scheme';
	const display_text_color_option_name = 'goals_for_patreon_text_color';
	const display_text_custom_color_option_name = 'goals_for_patreon_text_custom_color';
	const display_tick_color_option_name = 'goals_for_patreon_tick_color';
	const display_tick_custom_color_option_name = 'goals_for_patreon_custom_tick_color';
	const display_progress_bar_color_option_name = 'goals_for_patreon_progress_bar_color';
	const display_progress_bar_custom_color_option_name = 'goals_for_patreon_progress_bar_custom_color';
	const display_progress_bar_custom_hilight_color_option_name = 'goals_for_patreon_progress_bar_custom_hilight_color';
	const display_background_color_option_name = 'goals_for_patreon_background_color';
	const display_background_custom_color_option_name = 'goals_for_patreon_background_custom_color';
	const display_background_custom_hilight_color_option_name = 'goals_for_patreon_background_custom_hilight_color';
	
	const option_value_default = 'default';
	const option_value_inherit = 'inherit';
	const option_value_custom = 'custom';

	const available_goals_option_name = 'goals_for_patreon_available_goals';
	
	private static $defaults = array(
		self::display_progress_bar_custom_color_option_name => '#C80000',
		self::display_progress_bar_custom_hilight_color_option_name => '#FF0000',
		self::display_background_custom_color_option_name => '#B4B4B4',
		self::display_background_custom_hilight_color_option_name => '#E0E0E0',
	);

	public static function init() {
		new Options();
	}

	public function __construct() {
		add_action('admin_menu', array($this, 'admin_menu'));
		add_action('admin_init', array($this, 'register_settings'));
		add_action('admin_enqueue_scripts', array($this, 'enqueueStuff'));
	}

	public function enqueueStuff() {
		wp_register_script('gtfp-goal-tracker-options-js', plugins_url('js/options.js', dirname(__FILE__)), array('wp-color-picker'), false, true);
		wp_enqueue_script('gtfp-goal-tracker-options-js');

		wp_register_style('gtfp-patreon-campaign-options-style', plugins_url('css/options.css', dirname(__FILE__)));
		wp_enqueue_style('gtfp-patreon-campaign-options-style');
		wp_enqueue_style('wp-color-picker');
	}
	
	public function admin_menu() {
		$this->page = add_options_page(
			__('Goal Tracker for Patreon', 'textdomain'),
			__('Goal Tracker for Patreon', 'textdomain'),
			'manage_options',
			self::page_slug,
			array($this, 'print_settings_page')
		);
	}

	public function register_settings() {
		register_setting(self::gtgp_options_field_group_name, self::client_id_option_name);
		register_setting(self::gtgp_options_field_group_name, self::client_secret_option_name);
		register_setting(self::gtgp_options_field_group_name, self::access_token_option_name);
		register_setting(self::gtgp_options_field_group_name, self::refresh_token_option_name);

		add_settings_section(
			self::creator_info_section_name,
			__('Patreon Creator Info', 'textdomain'),
			array($this, 'print_user_info_section_html'),
			self::page_slug
		);

		add_settings_field(
			'patreon_client_id_field',
			'Client ID',
			array($this, 'print_client_id_html'),
			self::page_slug,
			self::creator_info_section_name
		);

		add_settings_field(
			'patreon_client_secret_field',
			'Client Secret',
			array($this, 'print_client_secret_html'),
			self::page_slug,
			self::creator_info_section_name
		);

		add_settings_field(
			'patreon_creator_access_key_field',
			'Creator Access Token',
			array($this, 'print_creator_access_key_html'),
			self::page_slug,
			self::creator_info_section_name
		);

		add_settings_field(
			'patreon_refresh_token_field',
			'Creator Refresh Token',
			array($this, 'print_refresh_token_html'),
			self::page_slug,
			self::creator_info_section_name
		);

		register_setting(self::gtgp_options_field_group_name, self::display_color_scheme_option_name);
		register_setting(self::gtgp_options_field_group_name, self::display_text_color_option_name);
		register_setting(self::gtgp_options_field_group_name, self::display_text_custom_color_option_name);
		register_setting(self::gtgp_options_field_group_name, self::display_tick_color_option_name);
		register_setting(self::gtgp_options_field_group_name, self::display_tick_custom_color_option_name);
		register_setting(self::gtgp_options_field_group_name, self::display_progress_bar_color_option_name);
		register_setting(self::gtgp_options_field_group_name, self::display_progress_bar_custom_color_option_name);
		register_setting(self::gtgp_options_field_group_name, self::display_progress_bar_custom_hilight_color_option_name);
		register_setting(self::gtgp_options_field_group_name, self::display_background_color_option_name);
		register_setting(self::gtgp_options_field_group_name, self::display_background_custom_color_option_name);
		register_setting(self::gtgp_options_field_group_name, self::display_background_custom_hilight_color_option_name);

		add_settings_section(
			self::display_options_section_name,
			__('Widget Display Options', 'textdomain'),
			array($this, 'print_display_settings_section_html'),
			self::page_slug
		);

		add_settings_field(
			'patreon_dislay_color_scheme_field',
			'Color Scheme: ',
			array($this, 'print_color_scheme_html'),
			self::page_slug,
			self::display_options_section_name
		);

		add_settings_field(
			'patreon_dislay_select_text_color_field',
			'Text color:',
			array($this, 'print_select_text_color_html'),
			self::page_slug,
			self::display_options_section_name
		);

		add_settings_field(
			'patreon_dislay_select_tick_color_field',
			'Tickmark color:',
			array($this, 'print_select_tick_color_html'),
			self::page_slug,
			self::display_options_section_name
		);

		add_settings_field(
			'patreon_dislay_progress_bar_colors_field',
			'Progress bar colors:',
			array($this, 'print_select_progress_bar_colors_html'),
			self::page_slug,
			self::display_options_section_name
		);

		add_settings_field(
			'patreon_dislay_background_colors_field',
			'Progress bar background colors:',
			array($this, 'printSelectBackgroundColorsHtml'),
			self::page_slug,
			self::display_options_section_name
		);

	}

	public function print_settings_page() {
		echo '<div class="wrap"><h2>Goal Tracker for Patreon Options Page</h2>';

		echo '<div class="options-message">';
		echo '<p style="margin-top: 0;">While I do not accept payment for this plugin, supporting my literary endeavors would be a lovely way to say thank you.</p>';
		echo '<div style="text-align: center;">';
		echo '<a href="https://www.patreon.com/bePatron?u=2933163&redirect_uri=http%3A%2F%2Fknives-the-novel.net%2Fthanks%2F"><img height="40" width="204" src="https://s3-us-west-1.amazonaws.com/widget-images/become-patron-widget-medium%402x.png"></a>';
		echo '</div>';
		echo '</div>';

		echo '<form method="post" action="options.php">';

		settings_fields(self::gtgp_options_field_group_name);
		do_settings_sections(self::page_slug);

		echo '<p class="submit"><input type="submit" class="button-primary" value="Save Changes") /></p>';

		echo '</form></div><!--END wrap-->';
	}

	public function print_user_info_section_html() {
		$html = '<p>To get your creator access token, log into your Patreon account and go to <a href="https://www.patreon.com/platform/documentation/clients">https://www.patreon.com/platform/documentation/clients</a> to register yourself.</p>';
		$html .= '<p><strong>Important:</strong> Even if you have already filled out the form for another Patreon plugin, you will need to do it again. Each plugin needs its own set of tokens or they will trip over each other.</p>';
		$html .= "<p>For App name, you can use the name of your WordPress site. For description, a description of your site is fine. I'm not sure if privacy policy and terms of service URL's are required, but on my site I made quickie pages that said I would never ever share info with anyone, and that I reserved the right to ban people who are jerks.</p>";
		$html .= "<p>Once you get all that stuff filled out, you will see a Client ID, a Client Secret, Creator's Access Token, and Creator's Refresh Token. Put each in its appropriate place below. When you save, you will see a little message over on the side with your creator id in it. That shows that your info was correct and you have successfully connected with Patreon.</p>";

		$html .= '<div class="options-message">';
		$cid = get_option(self::creator_id_option_name, '');
		$html .= '<strong>Creator Id:</strong> ';
		$html .= (empty($cid) ? 'not yet set' : $cid);
		$html .= '</div>';

		echo $html;

	}

	public function print_client_id_html() {
		echo '<input class="gtfp-option-fat" type="text" name="' . self::client_id_option_name . '" value="' . get_option(self::client_id_option_name) . '">';
	}

	public function print_client_secret_html() {
		echo '<input class="gtfp-option-fat" type="text" name="' . self::client_secret_option_name . '" value="' . get_option(self::client_secret_option_name) . '">';
	}

	public function print_creator_id_html() {
		$cid = get_option(self::creator_id_option_name, '');
		echo empty($cid) ? '(will update in automatically)' : $cid;
	}

	public function print_refresh_token_html() {
		echo '<input class="gtfp-option-wide type="text" name="' . self::refresh_token_option_name . '" value="' . get_option(self::refresh_token_option_name) . '">';
	}

	public function print_creator_access_key_html() {
		echo '<input class="gtfp-option-wide id="patreon_creator_access_key_field" type="text" name="' . self::access_token_option_name . '" value="' . get_option(self::access_token_option_name) . '">';
	}

	public function print_display_settings_section_html() {
		echo '<p>Choose a color setting</p>';
	}

	public function print_color_scheme_html() {
		$name = self::display_color_scheme_option_name;
		$currentValue = get_option($name);

		$html = '<ul><li>';
		$html .= '<input type="radio" id="'.$name.'_dark" name="'.$name.'" value="dark"'.($currentValue == 'dark' ? ' checked' : '').'>';
		$html .= '<label for="'.$name.'_dark">Dark background</label>';
		$html .= '</li><li>';
		$html .= '<input type="radio" id="'.$name.'_light" name="'.$name.'" value="light"'.($currentValue == 'light' ? ' checked' : '').'>';
		$html .= '<label for="'.$name.'_light">Light background</label>';
		$html .= '</li></ul>';

		echo $html;
	}
	
	public function print_select_text_color_html() {
		$this->printDICChoiceHTML(self::display_text_color_option_name, self::display_text_custom_color_option_name, 'Get from theme');
	}

	public function print_select_tick_color_html() {
		$this->printDICChoiceHTML(self::display_tick_color_option_name, self::display_tick_custom_color_option_name, 'Same as text color');
	}
	
	public function print_select_progress_bar_colors_html() {
		$this->printDefaultOrTwoColorSelectHtml(
			self::display_progress_bar_color_option_name,
			self::display_progress_bar_custom_color_option_name,
			self::display_progress_bar_custom_hilight_color_option_name
		);
	}

	public function printSelectBackgroundColorsHtml() {
		$this->printDefaultOrTwoColorSelectHtml(
			self::display_background_color_option_name,
			self::display_background_custom_color_option_name,
			self::display_background_custom_hilight_color_option_name
		);
	}
	
	private function printDICChoiceHTML($selectOptionName, $customOptionName, $inheritLabelText) {
		$currentSelectValue = get_option($selectOptionName, self::option_value_default);
		$customValue = get_option($customOptionName, '#000000');

		$html = '<ul><li>';
		$html .= '<input type="radio" class="d-i-c-choice" id="'.$selectOptionName.'_default" name="'.$selectOptionName.'" value="'.self::option_value_default.'"'.($currentSelectValue == self::option_value_default ? ' checked' : '').'>';
		$html .= '<label for="'.$selectOptionName.'_dark">Default from color scheme</label>';
		$html .= '</li><li>';
		$html .= '<input type="radio" class="d-i-c-choice" id="'.$selectOptionName.'_inherit" name="'.$selectOptionName.'" value="'.self::option_value_inherit.'"'.($currentSelectValue == self::option_value_inherit ? ' checked' : '').'>';
		$html .= '<label for="'.$selectOptionName.'_light">'.$inheritLabelText.'</label>';
		$html .= '</li><li>';
		$html .= '<input type="radio" class="d-i-c-choice" id="'.$selectOptionName.'_custom" name="'.$selectOptionName.'" value="'.self::option_value_custom.'"'.($currentSelectValue == self::option_value_custom ? ' checked' : '').'>';
		$html .= '<label for="'.$selectOptionName.'_light">Custom color</label>';

		$html .= '<input type="text" name="'.$customOptionName.'" value="'.$customValue.'" class="gtfp-color-field">';
		$html .= '</li></ul>';
		
		echo $html;
	}

	private function printDefaultOrTwoColorSelectHtml($selectOptionName, $baseOptionName, $hilightOptionName) {
		$currentSelectValue = get_option($selectOptionName, self::option_value_default);
		$baseOptionValue = get_option($baseOptionName, self::$defaults[$baseOptionName]);
		$hilightOptionValue = get_option($hilightOptionName, self::$defaults[$hilightOptionName]);

		$html = '<ul><li>';
		$html .= '<input type="radio" class="d-i-c-choice" id="'.$toggleOptionName.'_default" name="'.$selectOptionName.'" value="'.self::option_value_default.'"'.($currentSelectValue == self::option_value_default ? ' checked' : '').'>';
		$html .= '<label for="'.$name.'_dark">Default from color scheme</label>';
		$html .= '</li><li>';
		$html .= '<input type="radio" class="d-i-c-choice" id="'.$toggleOptionName.'_custom" name="'.$selectOptionName.'" value="'.self::option_value_custom.'"'.($currentSelectValue == self::option_value_custom ? ' checked' : '').'>';
		$html .= '<label for="'.$toggleOptionName.'_light">Custom colors:</label>';
		$html .= '</li><li>';
		$html .= '<label class="two-color-choice" for="'.$baseOptionName.'">Base color:</label>';
		$html .= '<input type="text" name="'.$baseOptionName.'" value="'.$baseOptionValue.'" class="gtfp-color-field">';
		$html .= '<label class="two-color-choice" for="'.$hilightOptionName.'">Hilight color:</label>';
		$html .= '<input type="text" name="'.$hilightOptionName.'" value="'.$hilightOptionValue.'" class="gtfp-color-field">';
		$html .= '</li></ul>';
		
		echo $html;
	}
}

Options::init();
